"""File:  CanvasTest.py  Test the Canvas widget and mouse events"""
from Tkinter import *
import sys

class Application(Frame):
    def __init__(self, master=None):
        Frame.__init__(self, master)
        self.master.title("Canvas and Mouse-events Example")
        
        self._dragging = False  # remembers if mouse button pressed
        self._currentShapeID = None

        self.grid()
        self._createWidgets()

    def _createWidgets(self):
        """Create GUI widgets"""
        self.shapeMenubutton = Menubutton(self, text="Shape", relief=RAISED)
        self.shapeMenubutton.grid(row=0, column=0, sticky=N)
        
        self.shapeMenubutton.menu = Menu(self.shapeMenubutton, tearoff=0)
        self.shapeMenubutton["menu"] = self.shapeMenubutton.menu
        self._shapeVar = StringVar()   # type of shape 
        self.shapeMenubutton.menu.add_radiobutton( label="oval", command=self._shapeSelection,
                                                   value = "circle", variable=self._shapeVar)
        self.shapeMenubutton.menu.add_radiobutton( label="rectangle",
                                                   command=self._shapeSelection,
                                                   value = "top_left_corner",
                                                   variable=self._shapeVar)
        self._shapeVar.set("circle")

        self._myCanvas = Canvas(self, cursor=self._shapeVar.get() ,borderwidth=4,
                               height='5i', width='7i')
        self._myCanvas.grid(row=1, column=0, columnspan=2)

        self._myCanvas.bind("<Button-1>", self._startShape)
        self._myCanvas.bind("<Motion>", self._sizeShape)
        self._myCanvas.bind("<ButtonRelease-1>", self._endShape)
        
        self.doneButton = Button(self, text='Click when done', command=self._doneSelections)
        self.doneButton.grid(row=3, column=0, columnspan=2,sticky=W+E)

    def _shapeSelection(self):
        """When shape radiobutton selected set the canvas cursor"""
        print "Shape selected:", self._shapeVar.get()
        self._myCanvas["cursor"] = self._shapeVar.get()

    def _startShape(self, event):
        """When left mouse button pressed, create the selected shape and set _dragging flag"""
        print "startShape:  shape =", self._shapeVar.get(),"at x =", event.x, "y =", event.y
        self._dragging = True
        if self._shapeVar.get() == "circle":
            self._currentShapeID = self._myCanvas.create_oval(event.x, event.y, event.x, event.y)
        elif self._shapeVar.get() == "top_left_corner":
            self._currentShapeID = self._myCanvas.create_rectangle(event.x, event.y,
                                                                 event.x, event.y)
        
    def _sizeShape(self, event):
        """When mouse is moved and dragging is True, create a new shape of the correct size."""
        if self._dragging:
            oldCoords = self._myCanvas.coords(self._currentShapeID)
            self._myCanvas.delete(self._currentShapeID)
            if self._shapeVar.get() == "circle":
                self._currentShapeID = self._myCanvas.create_oval(oldCoords[0], oldCoords[1],
                                                                event.x, event.y)
            elif self._shapeVar.get() == "top_left_corner":
                self._currentShapeID = self._myCanvas.create_rectangle(oldCoords[0], oldCoords[1],
                                                                     event.x, event.y)

    def _endShape(self, event):
        """When mouse button released, set _dragging flag to False"""
        self._dragging = False
    
    def _doneSelections(self):
        """When done button is clicked, exit and save drawing to encapsulated postscript file"""
        print "Done:  Drawing written to file 'SimpleDrawOut.eps'"
        self._myCanvas.postscript(file="SimpleDrawOut.eps")
        sys.exit

def main():
    """Instantiate and pop up the window."""
    Application().mainloop()

main()
